/*
	*
	* Copyright 2020 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project ( https://gitlab.com/desq/ )
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <cstdio>

#include <QtCore>
#include <QtDBus>

#include "DesQShellManager.hpp"

int main( int argc, char *argv[] ) {

	QApplication app( argc, argv );

    app.setOrganizationName( "DesQ" );
    app.setApplicationName( "DesQ Shell Manager" );
    app.setApplicationVersion( "1.0.0" );

    QCommandLineParser parser;
    parser.addHelpOption();         // Help
    parser.addVersionOption();      // Version

	parser.addOption( { { "d", "daemon" }, "Run this application as a daemon", "" } );
	parser.addOption( { "no-daemon", "Don't run this application as a daemon", "" } );

    parser.process( app );

	DesQShellManager manager;

	/* Another instance already exists? */
	if ( not manager.isServiceRunning() and not manager.isObjectRegistered() ) {
		qDebug() << "It appears that another instance of desq-shell is already active.";
		qDebug() << "You may use dbus to communicate with it. (Service Name: org.DesQ.Shell)";

		return 1;
	}

    /* Some one else has taken this service!! Grrr... */
	else if ( not manager.isServiceRunning() ) {
		qDebug() << "Unable to start the service 'org.DesQ.Shell'.";
        qDebug() << "Please ensure that another instance of this app is not running elsewhere.";
		return 1;
	}

    /* Something went wrong... */
	else if ( not manager.isObjectRegistered() ) {
		qDebug() << "Unable to register the object '/org/DesQ/Shell'.";
        qDebug() << "Please ensure that another instance of this app is not running elsewhere.";
		return 1;
	}

    /* All is well. */
	else {
		qDebug() << "Service org.DesQ.Shell running...";
		qDebug() << "Object /org/DesQ/Shell registered...";
		qDebug() << "Beginning Shell Management";
	}

	manager.startManagement();

    return app.exec();
};
